<?php

if ( ! defined( 'ABSPATH' ) ) {
    exit;
}

/*
* creating a function to create our CPT
*/
  
function gdprmap_custom_post_type() {
  
// set UI labels for Custom Post Type
    $labels = array(
        'name'                => _x( 'GDPR Map', 'Post Type General Name', 'gdprmap' ),
        'singular_name'       => _x( 'GDPR Maps', 'Post Type Singular Name', 'gdprmap' ),
        'menu_name'           => __( 'GDPR Maps', 'gdprmap' ),
        'parent_item_colon'   => __( 'Parent Maps', 'gdprmap' ),
        'all_items'           => __( 'All Maps', 'gdprmap' ),
        'view_item'           => __( 'View Maps', 'gdprmap' ),
        'add_new_item'        => __( 'Add New Maps', 'gdprmap' ),
        'add_new'             => __( 'Add New', 'gdprmap' ),
        'edit_item'           => __( 'Edit Maps', 'gdprmap' ),
        'update_item'         => __( 'Update Maps', 'gdprmap' ),
        'search_items'        => __( 'Search Maps', 'gdprmap' ),
        'not_found'           => __( 'Not Found', 'gdprmap' ),
        'not_found_in_trash'  => __( 'Not found in Trash', 'gdprmap' ),
    );
      
// set other options for Custom Post Type
      
    $args = array(
        'label'               => __( 'GDPR Maps', 'gdprmap' ),
        'description'         => __( 'GDPR compliant maps', 'gdprmap' ),
        'labels'              => $labels,
        /* A hierarchical CPT is like Pages and can have
        * Parent and child items. A non-hierarchical CPT
        * is like Posts.
        */
        'hierarchical'        => false,
        'public'              => false,
        'show_ui'             => true,
        'show_in_menu'        => true,
        'show_in_nav_menus'   => false,
        'show_in_admin_bar'   => true,
        'menu_position'       => 25,
        'can_export'          => true,
        'has_archive'         => true,
        'exclude_from_search' => true,
        'publicly_queryable'  => false,
        'capability_type'     => 'post',
        'show_in_rest' => false,
        'supports' => array('title')
  
    );
      
    // Registering your Custom Post Type
    register_post_type( 'gdprmap', $args );

    remove_post_type_support( 'gdprmap', 'comments' );
  
}
  
/* Hook into the 'init' action so that the function
* Containing our post type registration is not 
* unnecessarily executed. 
*/
  
add_action( 'init', 'gdprmap_custom_post_type', 0 );



/**
 * Woocommerce Address Autocomplete
**/
function gdprmap_include_autocomplete_js() {

    if ( class_exists( 'woocommerce' ) ) {

        // check if options is activated and the cart page is loaded
        if( is_checkout()  ) {
            wp_enqueue_script('gdpr-map-woo-autocomplete', plugin_dir_url( __DIR__ ) . 'js/address-autocomplete.js', array('jquery'));
            wp_enqueue_style('gdpr-map-front', plugin_dir_url( __DIR__ ) . 'css/style.css');
        }

    }

}
add_action('wp_enqueue_scripts', 'gdprmap_include_autocomplete_js');


/**
 * include JS and CSS
 */
function gdprmap_include_js_css() {


    // include CSS
    wp_enqueue_style('gdpr-map', plugin_dir_url( __DIR__ ) . 'includes/vendor/maplibre-gl/maplibre-gl.css');

    // include JS
    //wp_enqueue_script('gdpr-map', plugin_dir_url( __DIR__ ) . 'includes/vendor/maplibre-gl/csp/maplibre-gl-csp.js');
    //wp_add_inline_script( 'gdpr-map', 'maplibregl.workerUrl = "/wp-content/plugins/gdpr-map/includes/vendor/maplibre-gl/csp/maplibre-gl-csp-worker.js"; maplibregl.workerCount = 4; maplibregl.prewarm();' );
    wp_enqueue_script('gdpr-map', plugin_dir_url( __DIR__ ) . 'includes/vendor/maplibre-gl/maplibre-gl.js');

    $options = get_option( 'gdprmap_options' );

    wp_localize_script('gdpr-map', 'WPURLS', array( 'gdprmapurl' => plugin_dir_url( __DIR__ ), 'gdprmap_api_key' => $options['api_key'] ));

    wp_localize_script('gdpr-map', 'gdprmap_all_maps', (object) array() );

    wp_enqueue_script('gdpr-map-geocoder', plugin_dir_url( __DIR__ ) . 'js/geocoding.js', array('jquery', 'gdpr-map') );    

    wp_enqueue_script('gdpr-map-default-icon', plugin_dir_url( __DIR__ ) . 'js/default-icon.js', array('jquery', 'gdpr-map') );
}



/**
 * extend gdprmaps table
**/

add_filter( 'manage_edit-gdprmap_columns', 'gdprmap_add_new_maps_admin_list_column' );
function gdprmap_add_new_maps_admin_list_column( $columns ) {

    // Define columns
    $columns['gdprmap_options'] = 'Shortcode';
    return $columns;
}


/**
 * location list output
**/

add_action( 'manage_gdprmap_posts_custom_column', 'gdprmap_add_new_gdprmap_admin_list_column_content', 10, 2 ); 
function gdprmap_add_new_gdprmap_admin_list_column_content( $column, $post_id ) {


    // shortcode
    if ( 'gdprmap_options' === $column ) {

        ?>

        <div class="gdprmap-selectable" style="font-size: 14px;cursor: pointer;padding: 5px 10px;width: 150px;text-align: center;border: 1px solid grey;display: inline-block;margin-right: 10px;" id="gdprmap-selectable<?php echo $post_id; ?>">[gdprmap id="<?php echo $post_id; ?>"]</div>
        <div style="cursor: pointer; display: inline-block;top: 10px;position: relative;" id="gdprmap-copyButton<?php echo $post_id; ?>" class="copy-button" title="Copy to clipboard">
            <svg id="gdprmap-copySVG<?php echo $post_id; ?>" style="border-radius: 5px;transition: all 0.2s ease;" fill="#000000" width="30px" height="30px" viewBox="0 0 24 24" xmlns="http://www.w3.org/2000/svg">
              <path d="M8,7 L8,8 L6.5,8 C5.67157288,8 5,8.67157288 5,9.5 L5,18.5 C5,19.3284271 5.67157288,20 6.5,20 L13.5,20 C14.3284271,20 15,19.3284271 15,18.5 L15,17 L16,17 L16,18.5 C16,19.8807119 14.8807119,21 13.5,21 L6.5,21 C5.11928813,21 4,19.8807119 4,18.5 L4,9.5 C4,8.11928813 5.11928813,7 6.5,7 L8,7 Z M16,4 L10.5,4 C9.67157288,4 9,4.67157288 9,5.5 L9,14.5 C9,15.3284271 9.67157288,16 10.5,16 L17.5,16 C18.3284271,16 19,15.3284271 19,14.5 L19,7 L16.5,7 C16.2238576,7 16,6.77614237 16,6.5 L16,4 Z M20,6.52797748 L20,14.5 C20,15.8807119 18.8807119,17 17.5,17 L10.5,17 C9.11928813,17 8,15.8807119 8,14.5 L8,5.5 C8,4.11928813 9.11928813,3 10.5,3 L16.4720225,3 C16.6047688,2.99158053 16.7429463,3.03583949 16.8535534,3.14644661 L19.8535534,6.14644661 C19.9641605,6.25705373 20.0084195,6.39523125 20,6.52797748 Z M17,6 L18.2928932,6 L17,4.70710678 L17,6 Z M11.5,13 C11.2238576,13 11,12.7761424 11,12.5 C11,12.2238576 11.2238576,12 11.5,12 L13.5,12 C13.7761424,12 14,12.2238576 14,12.5 C14,12.7761424 13.7761424,13 13.5,13 L11.5,13 Z M11.5,11 C11.2238576,11 11,10.7761424 11,10.5 C11,10.2238576 11.2238576,10 11.5,10 L16.5,10 C16.7761424,10 17,10.2238576 17,10.5 C17,10.7761424 16.7761424,11 16.5,11 L11.5,11 Z M11.5,9 C11.2238576,9 11,8.77614237 11,8.5 C11,8.22385763 11.2238576,8 11.5,8 L16.5,8 C16.7761424,8 17,8.22385763 17,8.5 C17,8.77614237 16.7761424,9 16.5,9 L11.5,9 Z"/>
            </svg>
        </div>

        <script>
            document.getElementById('gdprmap-selectable<?php echo $post_id; ?>').addEventListener('click', function(event) {
                const range = document.createRange();
                range.selectNodeContents(this);
                const selection = window.getSelection();
                selection.removeAllRanges();
                selection.addRange(range);
            });

            document.getElementById('gdprmap-copyButton<?php echo $post_id; ?>').addEventListener('click', function() {
                const textElement = document.getElementById('gdprmap-selectable<?php echo $post_id; ?>');
                const range = document.createRange();
                range.selectNodeContents(textElement);
                const selection = window.getSelection();
                selection.removeAllRanges();
                selection.addRange(range);



                try {

                    document.execCommand('copy');

                    document.getElementById('gdprmap-copySVG<?php echo $post_id; ?>').style['background-color'] = 'green';
                    document.getElementById('gdprmap-copySVG<?php echo $post_id; ?>').style.fill = '#ffffff';

                    setTimeout(() => {
                        document.getElementById('gdprmap-copySVG<?php echo $post_id; ?>').style['background-color'] = '';
                        document.getElementById('gdprmap-copySVG<?php echo $post_id; ?>').style.fill = '#000000';
                    }, 1500);

                } catch (err) {
                    console.log('Failed to copy text');
                }

                // Entferne die Auswahl nach dem Kopieren
                selection.removeAllRanges();
            });
        </script>

        <?php
    }

}


/**
 * enable geojson upload
**/
function gdprmap_add_file_types_to_uploads($file_types){

    $file_types['geojson'] = 'application/json';
    return $file_types;
}
add_action('upload_mimes', 'gdprmap_add_file_types_to_uploads');


/*
 * The TinyMCE "Code" button is not included with WP by default
 */
function gdprmap_mce_external_plugins($plugins) {   

    $plugins['code'] = plugin_dir_path( __FILE__ ) . '/../js/tinymce.code.plugin.min.js';
    return $plugins;
}
add_filter('mce_external_plugins', 'gdprmap_mce_external_plugins');