<?php

if ( ! defined( 'ABSPATH' ) ) {
    exit;
}


/**
 * register menu item and page
 */
function gdprmap_admin_settings_page() {
    wp_enqueue_style('gdpr-map-admin', plugin_dir_url( __DIR__ ) . 'css/admin.css');
    wp_enqueue_script('gdpr-map-admin-wizard', plugin_dir_url( __DIR__ ) . 'js/admin-wizard.js', array('jquery') );
    add_options_page( 'GDPR map', 'GDPR map', 'manage_options', 'gdprmapsettings', 'gdprmap_settings_page' );
}
add_action( 'admin_menu', 'gdprmap_admin_settings_page' );


/**
 * settings page
 */
function gdprmap_settings_page() {

    ?>
    <h2>GDPR map settings</h2>
    <form action="options.php" method="post" class="gdprmap_setting_form">
        <?php
            settings_fields( 'gdprmap_options' );
            do_settings_sections( 'gdprmap_plugin_wizzard' );

            $options = get_option( 'gdprmap_options' );

            if( !empty($options['api_key']) ) :
        ?>
            <input name="gdprmap_options[mode]" type="hidden" id="gdprmap-mode" value="api" />
            <input name="submit" class="button button-primary" type="submit" value="<?php esc_attr_e( 'Save' ); ?>" />
        <?php else: ?>
            <input name="gdprmap_options[mode]" type="hidden" id="gdprmap-mode" value="register" />
            <input name="submit" class="button button-primary" type="submit" value="<?php esc_attr_e( 'Register' ); ?>" />
        <?php endif; ?>
    </form>
    <?php
}


/**
 * register settings
 */
function gdprmap_register_settings() {

    register_setting( 'gdprmap_options', 'gdprmap_options', 'gdprmap_options_validate' );
    add_settings_section( 'gdprmap_api_settings', 'API Settings', 'gdprmap_section_text', 'gdprmap_plugin_wizzard' );

    $options = get_option( 'gdprmap_options' );

    if( !empty($options['api_key']) ) {

        add_settings_field( 'gdprmap_setting_api_key', 'API Key', 'gdprmap_setting_api_key', 'gdprmap_plugin_wizzard', 'gdprmap_api_settings' );
        add_settings_field( 'gdprmap_setting_api_woocommerce', 'Woocommerce Address Autocomplete', 'gdprmap_setting_api_woocommerce', 'gdprmap_plugin_wizzard', 'gdprmap_api_settings' );

    }else{

        add_settings_field( 'gdprmap_setting_email', 'Email', 'gdprmap_setting_email', 'gdprmap_plugin_wizzard', 'gdprmap_api_settings' );
        add_settings_field( 'gdprmap_setting_api_key', 'API Key', 'gdprmap_setting_api_key', 'gdprmap_plugin_wizzard', 'gdprmap_api_settings' );
        add_settings_field( 'gdprmap_setting_change', '', 'gdprmap_setting_change', 'gdprmap_plugin_wizzard', 'gdprmap_api_settings' );
    }

}
add_action( 'admin_init', 'gdprmap_register_settings' ); 


/**
 * validate user input
 */
function gdprmap_options_validate( $input ) {

    $options_old = get_option( 'gdprmap_options' );
    $error = false;


    if( !empty($input['mode']) ) {
    
        // API key mode
        if( $input['mode'] == 'api' ) {

            // check if api key is valid
            $newinput['api_key'] = trim( $input['api_key'] );
            if( ctype_alnum( $newinput['api_key'] ) == false || strlen($newinput['api_key']) != 32 ) {
                $error = true;
                add_settings_error('prefix_messages', 'prefix_message', __('invalid API key', 'gdprmaps'), 'error');
            }

        }

            
        // register mode
        if( $input['mode'] == 'register' && empty($options_old['api_key']) ) {

            // check if checkboxes are checked
            if( $input['agb'] != 'agree' ) {
                $error = true;
                add_settings_error('prefix_messages', 'prefix_message', __('Please accept the terms and conditions and the privacy policy.'), 'error');
            }

            // check if valid email address
            if( !empty($input['email']) && !$error ) {

                if( filter_var($input['email'], FILTER_VALIDATE_EMAIL)  ) {

                    if( get_locale() == 'de_DE' || get_locale() == 'de_AT' || get_locale() == 'de_CH' ) {
                        $reg_locale = 'de_DE';
                    }else{
                        $reg_locale = 'en_US';
                    }


                    // get new api key
                    $json_data = file_get_contents('https://www.gdpr-map.eu/register-user/?email='.base64_encode($input['email']).'&url='.base64_encode(parse_url( get_bloginfo( 'url' ) )['host']).'&locale='.$reg_locale );


                    // decode response
                    $data = json_decode($json_data, true);

                    // check response
                    if( $data['status'] == 'error' ) {
                        $error = true;
                        add_settings_error('prefix_messages', 'prefix_message', $data['msg'], 'error');
                    }

                    if( $data['status'] == 'done' ) {

                        // save API key
                        $input['api_key'] = $data['apikey'];
                        add_settings_error('prefix_messages', 'prefix_message', esc_attr($data['msg']), 'success');

                        // set alert box with resend link
                    }
                }

            }elseif( empty($input['email']) && !$error ){
                $error = true;
                add_settings_error('prefix_messages', 'prefix_message', __('Invalid email adress.'), 'error');
            }
        }

    }


    if( $error ) {

        $input = $options_old;
    }


    return $input;
}


/**
 * section and fields
 */
function gdprmap_section_text() {
    $options = get_option( 'gdprmap_options' );

    if( empty($options['api_key']) ) {
        echo '<p>'.__('Register a new account or enter the API key.', 'gdprmap').'</p>';
    }else{
        echo '<p>'.__('Change API key.', 'gdprmap').'</p>';
    }
}

function gdprmap_setting_email() {
    $options = get_option( 'gdprmap_options' );
    if( empty($options['email']) ) {
        $this_email = get_option('admin_email', true);
        $options = [];
    }else{
        $this_email = $options['email'];
    }
    echo '<div class="gdprmap_setting_div"><input id="gdprmap_setting_email" name="gdprmap_options[email]" type="text" value="' . esc_attr($this_email) . '" /></div>';
    echo '<div class="gdprmap_setting_div">
        <label for="gdprmap_options[agb]">'.__('I agree to the <a href="https://www.gdpr-map.eu/en/terms-and-conditions/" target="_blank">terms and conditions</a> and the <a href="https://www.gdpr-map.eu/en/privacy-policy/" target="_blank">privacy policy</a>', 'gdprmap').' <span class="required">*</span></label><input type="checkbox" id="gdprmap_options[agb]" name="gdprmap_options[agb]" value="agree" '.checked($options['agb'], 'agree', false).' />
    </div>';
    echo '<input id="gdprmap_setting_resend" name="gdprmap_options[resend_link]" type="hidden" value="true" /></div>';

    /*
    echo '<div class="gdprmap_setting_div">
        <label for="gdprmap_options[privacy]">'.__('I agree to the <a href="https://www.gdpr-map.eu/en/privacy-policy/" target="_blank">privacy policy</a>', 'gdprmap').' <span class="required">*</span></label><input type="checkbox" id="gdprmap_options[privacy]" name="gdprmap_options[privacy]" value="agree" '.checked($options['privacy'], 'agree', false).' />
    </div>';
    */
}

function gdprmap_setting_api_key() {
    $options = get_option( 'gdprmap_options' );

    if( empty($options['api_key']) ) {
        echo "<input id='gdprmap_setting_api_key' name='gdprmap_options[api_key]' type='text' value='' />";
        echo "<script>jQuery(document).ready(function(){jQuery('.gdprmap_setting_form table tr:nth-child(2)').css('display', 'none');});</script>";
    }else{        
        echo "<input id='gdprmap_setting_api_key' name='gdprmap_options[api_key]' type='text' value='" . esc_attr( $options['api_key'] ) . "' />";

        // show resend link
        if( $options['resend_link'] == 'true' ) {

            // show this box once
            $options['resend_link'] = 'false';

            echo '<div id="gdprmap_setting_resend_activation"><p>'.__('Didn\'t receive an activation link? Request a new one here. Please note that you must click on the activation link first; otherwise, your map will not be activated and will not work.', 'gdprmap').'</p><button class="button button-1 button-primary" onclick="gdprmap_resend_activation_link(\''.esc_attr( $options['email'] ).'\');">'.__('Resend Activation Email', 'gdprmap').'</button><button class="button button-2 button-primary" onclick="return true;">'.__('Dismiss', 'gdprmap').'</button><span class="dashicons dashicons-warning"></span></div>';
        }


        // resend link
        if( $options['resend_active'] == 'true' ) {

            // show this box once
            $options['resend_active'] = 'false';
            update_option( 'gdprmap_options', $options );

            // get locale
            if( get_locale() == 'de_DE' || get_locale() == 'de_AT' || get_locale() == 'de_CH' ) {
                $reg_locale = 'de_DE';
            }else{
                $reg_locale = 'en_US';
            }


            // trigger new mail
            $json_data = file_get_contents('https://www.gdpr-map.eu/resend-doubloptin/?email='.base64_encode($options['resend_active_email']).'&locale='.$reg_locale );

            echo '<div id="gdprmap_setting_resend_activation"><p>'.__('If you don\'t receive a link within 15 minutes, please contact support at:<br /><br /><a href="mailto:info@gdpr-map.eu">info@gdpr-map.eu</a> | <a href="https://www.gdpr-map.eu/en/imprint" target="_blank">www.gdpr-map.eu</a>', 'gdprmap').'</p></div>';
        }
    }
}

function gdprmap_setting_api_woocommerce() {
    $options = get_option( 'gdprmap_options' );
    if( !empty($options['api_key']) ) {       
        echo "<input id='gdprmap_setting_api_woocommerce' name='gdprmap_options[api_woocommerce]' type='checkbox' value='true' ".checked($options['api_woocommerce'], 'true', false)." />";
    }
}

function gdprmap_setting_change() {
    echo '<a class="gdprmap-settings-toggle" href="#" onclick="gdprmap_toggle_wizard();return false;">'.__('enter API key directly', 'gdprmap').'</a><a class="gdprmap-settings-toggle" href="#" onclick="gdprmap_toggle_wizard();return false;" style="display: none;">'.__('register new account', 'gdprmap').'</a>';
}