/**
 * on document ready call init map
 */
jQuery(document).ready(function(){

	gdprmap_init_admin_map();
	gdprmap_add_edit_markers();

	
});


/**
 * init map
 */
var map;
var map_controll_geo;
var map_controll_full;
var map_controll_zoom;
var load_blocker = true;
var gdprmap_layers = [];
function gdprmap_init_admin_map() {

	// center
	if( jQuery('#gdprmap_center').val() != '' ) {

		var splice = jQuery('#gdprmap_center').val().split(',');
		var center = [ [parseFloat(splice[1])], [parseFloat(splice[0])] ];

	}else{

		var center = [ 14.5, 48.5 ];
	}

	// zoom
	var zoom = jQuery('#gdprmap_zoom').val();

	// pitch & bearing
	var bearing = jQuery('#gdprmap_bearing').val();
	var pitch = jQuery('#gdprmap_pitch').val();

	map = new maplibregl.Map({
	  container: 'gdprmap-edit',
	  center: center,
	  zoom: zoom,
	  pitch: pitch,
	  bearing: bearing,
	  minZoom: 1,
	  style: 'https://map02.gdpr-map.eu/styles/'+jQuery('#gdprmap_style').val()+'/style.json?key='+WPURLS.gdprmap_api_key,
	  attributionControl: false
	});


	gdprmap_all_maps[0] = map;

	map.on('movestart', function(){
		jQuery('.gdprmap-geolocation-autocomplete-tbl').css('display', 'none');
		jQuery('.gdprmap-geolocation-autocomplete').removeClass('open');
	});

	map.on('dragstart', function(){
		jQuery('.gdprmap-geolocation-autocomplete-tbl').css('display', 'none');
		jQuery('.gdprmap-geolocation-autocomplete').removeClass('open');
	});


	// Update selector settings
	jQuery('.gdprmap-style-selector > div').removeClass('selected');
	jQuery('.gdprmap-style-selector > div[data-style="'+jQuery('#gdprmap_style').val()+'"]').addClass('selected');


   	// double click zoom
   	if( jQuery('#gdprmap_doubleclick').prop('checked') ) {
   		map.doubleClickZoom.enable();
   	}else{
   		map.doubleClickZoom.disable();
   	}

   	// scroll zoom
   	if( !jQuery('#gdprmap_scroll').prop('checked') ) {
   		map.scrollZoom.enable();
   	}else{
   		map.scrollZoom.disable();
   	}

   	// add zoom and rotation controls to the map.
	if( !jQuery('#gdprmap_zoom_controll').prop('checked') ) {
    	map_controll_zoom = new maplibregl.NavigationControl();
		map.addControl(map_controll_zoom);
	}

  	// fullscreen
   	if( jQuery('#gdprmap_fullscreen_controll').prop('checked') ) {
   		map_controll_full = new maplibregl.FullscreenControl();
		map.addControl(map_controll_full);
   	}

    // Show Geolocator
    if( jQuery('#gdprmap_geolocator').prop('checked') ) {

	    map_controll_geo = new maplibregl.GeolocateControl({
	        positionOptions: {
	            enableHighAccuracy: true
	        },
	        trackUserLocation: true
	    });

		map.addControl(map_controll_geo);
	}




    // Attribution controll
    map.addControl(new maplibregl.AttributionControl({
		customAttribution: '<a href="https://www.gdpr-map.eu" target="_blank" title="GDPR map" aria-label="GDPR map">GDPR map</a> | <a href="https://www.openmaptiles.org/" target="_blank" title="OpenMapTiles" aria-label="OpenMapTiles">© OpenMapTiles</a> | <a href="https://www.openstreetmap.org/copyright" target="_blank" title="OpenStreetMap" aria-label="OpenStreetMap">© OpenStreetMap contributors</a>'
	}));

    map.on('moveend', (e) => {
    	jQuery("#gdprmap_center").val( map.getCenter().lat+","+map.getCenter().lng );
    	jQuery('#gdprmap_zoom').val(map.getZoom());
    	jQuery('#gdprmap_bearing').val(map.getBearing());
    	jQuery('#gdprmap_pitch').val(map.getPitch());
    });

    map.on('zoomend', (e) => {
    	jQuery('#gdprmap_zoom').val(map.getZoom());
    });

    map.on('pitchend', (e) => {
    	jQuery('#gdprmap_pitch').val(map.getPitch());
    });

    map.on('rotateend', (e) => {
    	jQuery('#gdprmap_bearing').val(map.getBearing());
    });
    

    // Activate Layer on map load
	map.on('load', async () => {
		

		geojson_marker.forEach(function(mrk, index) {

			var subsrc = mrk.source;
			var marker_min = 0;
			var default_offset = 25;

			// extend source properties when clutering
			if( mrk.showcluster != 'nocluster' ) {

				subsrc.cluster = true;
	            subsrc.clusterMaxZoom = 14;
	            subsrc.clusterRadius = 50;
	            marker_min = subsrc.clusterMaxZoom;
			}
		  	
		  	// load source
		  	map.addSource(mrk.sourceid, subsrc);


		  	// add cluster layers
		  	if( mrk.showcluster != 'nocluster' ) {

			  	// Cluster circles
			  	map.addLayer({
			        id: 'clusters'+mrk.imageid,
			        type: 'circle',
			        source: mrk.sourceid,
			        filter: ['has', 'point_count'],
			        paint: mrk.cluster,
			        maxzoom: marker_min
			    });


			  	// Cluster count
		        map.addLayer({
			        id: 'cluster-count'+mrk.imageid,
			        type: 'symbol',
			        source: mrk.sourceid,
			        filter: ['has', 'point_count'],
			        layout: {
			            'text-field': '{point_count_abbreviated}',
			            'text-font': ['Open Sans Regular'],
			            'text-size': 12
			        },
			        maxzoom: marker_min,
				    paint: {
				        'text-color': mrk.clustertxtcolor
				    }
			    });
			}

	        // push layer ID
		    gdprmap_layers.push({layerid: mrk.imageid, markerid: mrk.markerid, source: subsrc, sourceid: mrk.sourceid, clusterjson: mrk.cluster});


		    // zoom on click
		    map.on('click', 'clusters'+mrk.imageid, async (e) => {
		        const features = map.queryRenderedFeatures(e.point, {
		            layers: ['clusters'+mrk.imageid]
		        });
		        const clusterId = features[0].properties.cluster_id;
		        const zoom = await map.getSource(mrk.sourceid).getClusterExpansionZoom(clusterId);
		        map.easeTo({
		            center: features[0].geometry.coordinates,
		            zoom
		        });
		    });

		    // Change cursor to pointer
		    map.on('mouseenter', 'clusters'+mrk.imageid, () => {
		        map.getCanvas().style.cursor = 'pointer';
		    });
		    map.on('mouseleave', 'clusters'+mrk.imageid, () => {
		        map.getCanvas().style.cursor = '';
		    });



		  	// Add layer with markers
		  	if( mrk.type == "default" ) {

		  		// default marker
		  		gdprmap_create_default_marker(mrk.imageid, mrk.sourceid, mrk.imgcolor, marker_min, map);		  		


		  	}else{
		  		
		  		// Custom icon marker
		  		gdprmap_create_custom_marker(mrk.imageid, mrk.sourceid, mrk.imgurl, marker_min, map)

		  		// set new offet
		  		default_offset = parseInt(mrk.offset*0.61);
		  	}



		  	// click on markers
		  	map.on('click', mrk.imageid, (e) => {

	            const coordinates = e.features[0].geometry.coordinates.slice();

	            if( typeof e.features[0].properties.html != 'undefined' ) {

	            	const html = e.features[0].properties.html;

		            if( html.trim() != "" ) {
		            
			            while (Math.abs(e.lngLat.lng - coordinates[0]) > 180) {
			                coordinates[0] += e.lngLat.lng > coordinates[0] ? 360 : -360;
			            }

			            new maplibregl.Popup({offset: default_offset})
			                .setLngLat(coordinates)
			                .setHTML(
			                    html
			                )
			                .addTo(map);
			        }
	            }
	            
	        });

		    
		});

		load_blocker = false;
	});


   
}



/**
 * button manual positioning, zoom, pitch and bearing
 */
function gdprmap_center_map() {

	var splice = jQuery('#gdprmap_center').val().split(',');
	var pitch = jQuery('#gdprmap_pitch').val();
	var bearing = jQuery('#gdprmap_bearing').val();

	if( isNaN(parseFloat(splice[1])) || isNaN(parseFloat(splice[0])) ) {
		center = [[0], [0]];
		jQuery('#gdprmap_center').val('0,0');
	}else{
		var center = [ [parseFloat(splice[1])], [parseFloat(splice[0])] ];		
	}
	

	map.setZoom(jQuery('#gdprmap_zoom').val());
	map.setCenter(center);
	map.setPitch(parseInt(pitch));
	map.setBearing(parseInt(bearing));
}


/**
 * add enter event listener
 */
var enter_listen_items = [
'gdprmap_width',
'gdprmap_height',
'gdprmap_center',
'gdprmap_zoom',
'gdprmap_pitch',
'gdprmap_bearing' ];


enter_listen_items.forEach(function(val, ind){
	document.getElementById(val).addEventListener('keydown', function(event) {
        if (event.key === 'Enter') {
            event.preventDefault();
            gdprmap_center_map();
            return false;
        }
    });
});


/**
 * add new marker
 */
var markercnt = 0;
var marker_arr = [];
var editor_arr = [];
var popup_arr = [];
var geojson_marker = [];
function gdprmap_add_marker() {

	// create new row
	jQuery('#gdprmap-marker-tbl').prepend('<div id="gdpr-marker-'+markercnt+'" class="gdpr-marker">   <div class="gdpr-marker-left">      <input type="hidden" id="gdpr-marker-'+markercnt+'-lat" name="gdpr-marker-lat[]" />      <input type="hidden" id="gdpr-marker-'+markercnt+'-lng" name="gdpr-marker-lng[]" />      <input type="hidden" id="gdpr-marker-'+markercnt+'-geojson" name="gdpr-marker-geojson[]" value="" />      <input type="hidden" id="gdpr-marker-'+markercnt+'-icon" name="gdpr-marker-icon[]" value="" />      <div class="gdpr-marker-title">         <input type="text" id="gdpr-marker-'+markercnt+'-name" name="gdpr-marker-name[]" placeholder="Title" class="regular-text">      </div>                  <div class="gdpr-marker-info" id="gdpr-marker-info-'+markercnt+'">         <div class="gdpr-marker-info-custom-icon">            <input type="radio" name="gdpr-marker-custom-icon-'+markercnt+'" id="gdpr-marker-custom-icon-'+markercnt+'-default" class="gdpr-marker-custom-icon" value="true" checked /><label for="gdpr-marker-custom-icon-'+markercnt+'-default">use default icon</label>            <input type="radio" name="gdpr-marker-custom-icon-'+markercnt+'" id="gdpr-marker-custom-icon-'+markercnt+'" class="gdpr-marker-custom-icon" value="false" /><label for="gdpr-marker-custom-icon-'+markercnt+'">use custom icon</label>         </div>         <div class="gdpr-marker-info-colorpicker">            <span>Marker color</span><input type="text" value="#ef1717" class="gdpr-marker-info-colorfield-mrkcolor gdpr-marker-info-colorfield" name="gdpr-marker-color[]" data-default-color="#ef1717" />         </div>         <div class="gdpr-marker-info-icon-selector" style="display: none;">            <button class="button button-primary" onClick="gdprmap_custom_marker_style('+markercnt+');return false;">select icon</button><div class="gdpr-marker-info-icon"></div>         </div>         <div class="gdpr-marker-info-geojson-btn">            <input type="radio" name="gdpr-marker-use-geojson-'+markercnt+'" class="gdpr-marker-use-geojson" id="gdpr-marker-use-geojson-'+markercnt+'-html" value="true" checked /><label for="gdpr-marker-use-geojson-'+markercnt+'-html">use HTML</label>            <input type="radio" name="gdpr-marker-use-geojson-'+markercnt+'" class="gdpr-marker-use-geojson" id="gdpr-marker-use-geojson-'+markercnt+'" value="false" /><label for="gdpr-marker-use-geojson-'+markercnt+'">use geojson</label>   <div class="gdprmap-marker-force-reload"><button class="button button-primary button-large" onClick="gdprmap_jump_to_marker('+markercnt+', jQuery(this)); return false;">goto marker</button></div>      </div>         <div class="gdpr-marker-info-geojson-selector" style="display: none;">            <div class="gdpr-marker-info-geojson-selector-section">               <button class="button button-primary" onClick="gdprmap_upload_geojson('+markercnt+');return false;">select geojson</button><div class="gdpr-marker-info-geojson"></div>           </div>            <div class="gdpr-marker-info-geojson-selector-section-0">               <input type="radio" name="gdpr-marker-cluster-geojson-'+markercnt+'" id="gdpr-marker-cluster-geojson-'+markercnt+'-nocluster" class="gdpr-marker-cluster-geojson" value="false" checked /><label for="gdpr-marker-cluster-geojson-'+markercnt+'-nocluster">do not cluster</label>               <input type="radio" name="gdpr-marker-cluster-geojson-'+markercnt+'" id="gdpr-marker-cluster-geojson-'+markercnt+'" class="gdpr-marker-cluster-geojson" value="true" /><label for="gdpr-marker-cluster-geojson-'+markercnt+'">cluster items</label>            </div>            <div class="gdpr-marker-info-geojson-selector-section-1" style="display: none;"> <div class="gdpr-marker-info-geojson-selector-section-1-textcolor">                  <span>Counter text color</span><input type="text" value="#000000" class="gdpr-marker-cluster-geojson-textcolor gdpr-marker-info-colorfield" name="gdpr-marker-cluster-geojson-textcolor[]" data-default-color="#000000" />               </div>              <table class="wp-list-table widefat fixed striped">                  <thead>                     <tr>                        <th>No</th>                        <th>Active</th>                        <th>Color</th>                        <th>Radius</th>                        <th>Threshold</th>                     </tr>                  </thead>                  <tbody>                     <tr>                        <td>1</td>                        <td><input type="checkbox" name="gdpr-marker-cluster-geojson-active-1[]" class="gdpr-marker-cluster-geojson-active-1" value="1" checked /></td>                        <td><div class="gdpr-marker-cluster-geojson-color-wrapper"><input type="text" value="#729fd1" class="gdpr-marker-cluster-geojson-color gdpr-marker-info-colorfield" name="gdpr-marker-cluster-geojson-color-1[]" data-default-color="#729fd1" /></div></td>                        <td><input type="number" value="20" min="1" class="gdpr-marker-cluster-geojson-radius" name="gdpr-marker-cluster-geojson-radius-1[]" /></td>                        <td><input type="number" value="50" min="1" class="gdpr-marker-cluster-geojson-threshold" name="gdpr-marker-cluster-geojson-threshold-1[]" /></td>                     </tr>                     <tr>                        <td>2</td>                        <td><input type="checkbox" name="gdpr-marker-cluster-geojson-active-2[]" class="gdpr-marker-cluster-geojson-active-2" value="2" checked /></td>                        <td><div class="gdpr-marker-cluster-geojson-color-wrapper"><input type="text" value="#4169a1" class="gdpr-marker-cluster-geojson-color gdpr-marker-info-colorfield" name="gdpr-marker-cluster-geojson-color-2[]" data-default-color="#4169a1" /></div></td>                        <td><input type="number" value="30" min="1" class="gdpr-marker-cluster-geojson-radius" name="gdpr-marker-cluster-geojson-radius-2[]" /></td>                        <td><input type="number" value="250" min="1" class="gdpr-marker-cluster-geojson-threshold" name="gdpr-marker-cluster-geojson-threshold-2[]" /></td>                     </tr>                     <tr>                        <td>3</td>                        <td><input type="checkbox" name="gdpr-marker-cluster-geojson-active-3[]" class="gdpr-marker-cluster-geojson-active-3" value="3" /></td>                        <td><div class="gdpr-marker-cluster-geojson-color-wrapper"><input type="text" value="#2b3a67" class="gdpr-marker-cluster-geojson-color gdpr-marker-info-colorfield" name="gdpr-marker-cluster-geojson-color-3[]" data-default-color="#2b3a67" /></div></td>                        <td><input type="number" value="40" min="1" class="gdpr-marker-cluster-geojson-radius" name="gdpr-marker-cluster-geojson-radius-3[]" /></td>                        <td></td>                     </tr>                  </tbody>               </table>            </div>         </div>      </div>      <div class="gdpr-marker-buttons">         <button class="button button-primary del-button" onClick="jQuery(this).next().css(\'display\', \'inline-block\'); return false;">delete</button>                  <span style="display: none;">Are you sure? <span onClick="gdprmap_delete_marker('+markercnt+');return false;">yes</span> | <span onClick="jQuery(this).parent().css(\'display\', \'none\')">no</span></span>               </div>   </div>   <div class="gdpr-marker-right">      <textarea id="gdpr-marker-'+markercnt+'-html" name="gdpr-marker-html[]" placeholder="HTML"></textarea>   </div></div>');

	


	// change color picker default colors
	var defaultColor = '#ef1717';
	if( typeof gdprmap_markers[markercnt] != 'undefined' ) {
		
		// default marker color
		defaultColor = gdprmap_markers[markercnt].markercolor;
		jQuery('#gdpr-marker-'+markercnt+' .gdpr-marker-info-colorpicker .gdpr-marker-info-colorfield').val(defaultColor);
		jQuery('#gdpr-marker-'+markercnt+' .gdpr-marker-info-colorpicker .gdpr-marker-info-colorfield').attr('data-default-color', defaultColor);

		// cluster colors 
		var optcnt = 1;
		var defaultColorOPT;
		while( optcnt <= 3 ) {

			defaultColorOPT = gdprmap_markers[markercnt]["geojsonclustercolor"+optcnt];
			jQuery('#gdpr-marker-'+markercnt+' table tbody tr:nth-child('+optcnt+') .gdpr-marker-info-colorfield').val(defaultColorOPT);
			jQuery('#gdpr-marker-'+markercnt+' table tbody tr:nth-child('+optcnt+') .gdpr-marker-info-colorfield').attr('data-default-color', defaultColorOPT);
			optcnt++;
		}

		// cluster text color
		defaultColorText = gdprmap_markers[markercnt]['geojson-textcolor'];
		jQuery('#gdpr-marker-'+markercnt+' .gdpr-marker-cluster-geojson-textcolor').val(defaultColorText);
		jQuery('#gdpr-marker-'+markercnt+' .gdpr-marker-cluster-geojson-textcolor').attr('data-default-color', defaultColorText);
	}
	

	// invoke color picker
	jQuery('#gdpr-marker-'+markercnt+' .gdpr-marker-info-colorfield').wpColorPicker({change: function(event, ui){


		// LIVE preview for single markers
		var new_color = ui.color.toCSS();
		var marker_id = parseInt(jQuery(event.target).closest('.gdpr-marker-info').attr('id').replace('gdpr-marker-info-', ''));

		// save target value
		jQuery(event.target).val(new_color);



		if( marker_arr[marker_id] != 'geojson' && jQuery(event.target).hasClass('gdpr-marker-info-colorfield-mrkcolor') ) {

			// Default marker
			if( jQuery('[name="gdpr-marker-custom-icon-'+marker_id+'"]:checked').val() == "true" ) {

				// Default marker
				var default_marker = gdprmap_create_default_marker_svg(new_color).svg;

				marker_arr[marker_id].getElement().innerHTML = default_marker;
			}
		}

		// GeoJSON
		if( marker_arr[marker_id] == 'geojson' ) {

			// WE HAVE CLUSTERS
			// find layerID
			var layerid = "";
			gdprmap_layers.forEach(function(layer, ind){

				if(layer.markerid == marker_id) {
					layerid = layer.layerid;
					return true;
				}
				
			});


			if( layerid != "" ) {

				// Change default Marker
				if( jQuery('[name="gdpr-marker-custom-icon-'+marker_id+'"]:checked').val() == "true" && jQuery(event.target).hasClass('gdpr-marker-info-colorfield-mrkcolor') ) {

					
					var svgUrl = gdprmap_create_default_marker_svg(new_color).url;

					createImageBitmapFromSVG(svgUrl).then(imageBitmap => {

						if (map.hasImage('M' + layerid)) {
				        	// Aktualisiere das bestehende Bild
				        	map.updateImage('M' + layerid, imageBitmap);
				        	map.redraw();
					    } else {
					        // Füge das Bild hinzu, wenn es nicht existiert
					        map.addImage('M' + layerid, imageBitmap);
					        map.redraw();
					    }

					}).catch(error => {
					    console.error('Fehler beim Erstellen des ImageBitmap:', error);
					});
				    
				}

				if( !jQuery(event.target).hasClass('gdpr-marker-info-colorfield-mrkcolor') ) {

					// change cluster font color
					if( jQuery(event.target).hasClass('gdpr-marker-cluster-geojson-textcolor') ) {
						map.setPaintProperty('cluster-count'+layerid, 'text-color', new_color);
					}
					

					// Change cluster color
					if( jQuery(event.target).hasClass('gdpr-marker-cluster-geojson-color') ) {

						// Circle color
						var tmp_paint_color = [
						        'step',
						        ['get', 'point_count'],
						        jQuery('#gdpr-marker-'+marker_id+' .gdpr-marker-info-geojson-selector-section-1 table tr:nth-child(1) .gdpr-marker-cluster-geojson-color').val(),
						        parseInt(jQuery('#gdpr-marker-'+marker_id+' .gdpr-marker-info-geojson-selector-section-1 table tr:nth-child(1) .gdpr-marker-cluster-geojson-threshold').val()),
						        jQuery('#gdpr-marker-'+marker_id+' .gdpr-marker-info-geojson-selector-section-1 table tr:nth-child(2) .gdpr-marker-cluster-geojson-color').val()
						    ];

						
						if( jQuery('#gdpr-marker-'+marker_id+' .gdpr-marker-info-geojson-selector-section-1 table tr:nth-child(3) input[type="checkbox"]').prop('checked') == true ) {
							
							tmp_paint_color.push( parseInt(jQuery('#gdpr-marker-'+marker_id+' .gdpr-marker-info-geojson-selector-section-1 table tr:nth-child(2) .gdpr-marker-cluster-geojson-threshold').val()) );
							tmp_paint_color.push( jQuery('#gdpr-marker-'+marker_id+' .gdpr-marker-info-geojson-selector-section-1 table tr:nth-child(3) .gdpr-marker-cluster-geojson-color').val() );
							
						}


						// Circle radius
						var tmp_paint_radius = [
					        'step',
					        ['get', 'point_count'],
					        parseInt(parseInt(jQuery('#gdpr-marker-'+marker_id+' .gdpr-marker-info-geojson-selector-section-1 table tr:nth-child(1) .gdpr-marker-cluster-geojson-radius').val())),
					        parseInt(parseInt(jQuery('#gdpr-marker-'+marker_id+' .gdpr-marker-info-geojson-selector-section-1 table tr:nth-child(1) .gdpr-marker-cluster-geojson-threshold').val())),
					        parseInt(parseInt(jQuery('#gdpr-marker-'+marker_id+' .gdpr-marker-info-geojson-selector-section-1 table tr:nth-child(2) .gdpr-marker-cluster-geojson-radius').val())),			        
					    ]

					    if( jQuery('#gdpr-marker-'+marker_id+' .gdpr-marker-info-geojson-selector-section-1 table tr:nth-child(3) input[type="checkbox"]').prop('checked') == true ) {

					    	tmp_paint_radius.push( parseInt(jQuery('#gdpr-marker-'+marker_id+' .gdpr-marker-info-geojson-selector-section-1 table tr:nth-child(2) .gdpr-marker-cluster-geojson-threshold').val()) );
							tmp_paint_radius.push( parseInt(jQuery('#gdpr-marker-'+marker_id+' .gdpr-marker-info-geojson-selector-section-1 table tr:nth-child(3) .gdpr-marker-cluster-geojson-radius').val()) );
					    }
						

						map.setPaintProperty('clusters'+layerid, 'circle-color', tmp_paint_color);
						map.setPaintProperty('clusters'+layerid, 'circle-radius', tmp_paint_radius);
						
					}
				}
			}			
		}
	}});



	// create the popup
    const popup = new maplibregl.Popup({offset: 25});

    popup_arr.push(popup);


    var this_marker = markercnt;

    // tinymce init & popup text change listener
    editor_arr[markercnt] = new tinymce.Editor('gdpr-marker-'+markercnt+'-html', {
	  //selector: '#gdpr-marker-'+markercnt+'-html',
	  menubar: 'edit, format',
	  height: 240,
	  external_plugins: {
	    code: GDPRMAP.plugindir+'js/tinymce.code.plugin.min.js'
	  },
	  branding: false,
	  toolbar: 'undo, redo, bold, italic, underline, bullist, numlist, indent, outdent, link, unlink, code',
	  plugins: 'link, lists, code, colorpicker',
	  init_instance_callback: function(editor) {
	    editor.on('keyup', function(e) {
	    	if( popup_arr[this_marker].isOpen() ) {
	    		popup_arr[this_marker].remove();
	      		popup_arr[this_marker].setHTML(
			        tinymce.get('gdpr-marker-'+this_marker+'-html').getContent({format: 'html'})
			    );
			    popup_arr[this_marker].addTo(map);
			    tinymce.execCommand('mceFocus',false,'gdpr-marker-'+this_marker+'-html');

	    	}else{
	    		popup_arr[this_marker].setHTML(
			        tinymce.get('gdpr-marker-'+this_marker+'-html').getContent({format: 'html'})
			    );
	    	}
	      
	    });
	  }
	}, tinymce.EditorManager);

	editor_arr[markercnt].render();



    // create the marker
    var marker;
    if( typeof gdprmap_markers[markercnt] !== 'undefined' ) {

    	// Simple marker

    	if( gdprmap_markers[markercnt].usehtml == "true" ) {

    		// Custom Marker Icon with HTML Popup
	    	if( gdprmap_markers[markercnt].defaulticon == "false" && gdprmap_markers[markercnt].icon != "" ) {

	    		var icon_data = JSON.parse(gdprmap_markers[markercnt].icon);

	    		const geojson = {
			        'type': 'FeatureCollection',
			        'features': [
			            {
			                'type': 'Feature',
			                'properties': {
			                    'iconSize': [icon_data.width, icon_data.height],
			                    'iconUrl': icon_data.url
			                },
			                'geometry': {
			                    'type': 'Point',
			                    'coordinates': [gdprmap_markers[markercnt].lng, gdprmap_markers[markercnt].lat]
			                }
			            }
			        ]
			    };


			    // create a DOM element for the marker
		        const el = document.createElement('div');
		        el.className = 'custommarkericon';
		        el.style.backgroundImage = 'url('+geojson.features[0].properties.iconUrl+')';
		        el.style.width = `${geojson.features[0].properties.iconSize[0]}px`;
		        el.style.height = `${geojson.features[0].properties.iconSize[1]}px`;

		        marker = new maplibregl.Marker({element: el, draggable: true, id: 'marker-with-popup-'+markercnt})
		        .setLngLat(geojson.features[0].geometry.coordinates)
		        .setPopup(popup) // sets a popup on this marker
		        .addTo(map);

	    	}else{

	    		// Default Marker Icon when Icon is missing
	    		marker = new maplibregl.Marker({color: defaultColor, draggable: true, id: 'marker-with-popup-'+markercnt})
		        .setLngLat([gdprmap_markers[markercnt].lng, gdprmap_markers[markercnt].lat])
		        .setPopup(popup) // sets a popup on this marker
		        .addTo(map);
	    	}



	    }else if( gdprmap_markers[markercnt].usehtml == "false" && gdprmap_markers[markercnt].geojson != "" ){

	    	// check for cluster
	    	// #######################

	    	var this_paint = {
			    'circle-color': [
			        'step',
			        ['get', 'point_count'],
			        gdprmap_markers[markercnt].geojsonclustercolor1,
			        parseInt(gdprmap_markers[markercnt].geojsonthreshold1)
			    ],
			    'circle-radius': [
			        'step',
			        ['get', 'point_count'],
			        parseInt(gdprmap_markers[markercnt].geojsonradius1),
			        parseInt(gdprmap_markers[markercnt].geojsonthreshold1)				        
			    ]
			};

			if( gdprmap_markers[markercnt].geojsonactive2 == "2" ) {

				this_paint['circle-color'].push( gdprmap_markers[markercnt].geojsonclustercolor2 );
				this_paint['circle-radius'].push( parseInt(gdprmap_markers[markercnt].geojsonradius2) );
			}

			if( gdprmap_markers[markercnt].geojsonactive3 == "3" ) {
				
				this_paint['circle-color'].push( parseInt(gdprmap_markers[markercnt].geojsonthreshold2), gdprmap_markers[markercnt].geojsonclustercolor3 );
				this_paint['circle-radius'].push( parseInt(gdprmap_markers[markercnt].geojsonthreshold2), parseInt(gdprmap_markers[markercnt].geojsonradius3) );
			}


	    	var this_cluster_show = 'nocluster';
	    	if( gdprmap_markers[markercnt].clustergeojson == "true" ) {	    		

				this_cluster_show = 'cluster';

	    	}



	    	
	    	// Marker Cluster
	    	if( gdprmap_markers[markercnt].defaulticon == "false" && gdprmap_markers[markercnt].icon != "" ) {

	    		// With custom Icon
	    		var geojsondata = JSON.parse(gdprmap_markers[markercnt].geojson);		
	    		var icon_data = JSON.parse(gdprmap_markers[markercnt].icon);


	    		geojson_marker.push({markerid: markercnt, sourceid: 'geojson'+geojsondata.id, source: {
			        type: 'geojson',
			        data: geojsondata.url
			    }, imageid: 'marker-icon'+geojsondata.id,
				    imgurl: icon_data.url, type: 'customicon', offset: icon_data.height, cluster: this_paint, showcluster: this_cluster_show, clustertxtcolor: gdprmap_markers[markercnt]['geojson-textcolor']
				});

	    		// dummy marker
	    		marker = 'geojson';
			    

	    	}else{

	    		// With default Icon
	    		var geojsondata = JSON.parse(gdprmap_markers[markercnt].geojson);
	    		
	    		geojson_marker.push({markerid: markercnt, sourceid: 'geojson'+geojsondata.id, source: {
			        type: 'geojson',
			        data: geojsondata.url
			    }, imageid: 'marker-icon'+geojsondata.id,
			    	imgcolor: defaultColor,type: 'default', cluster: this_paint, showcluster: this_cluster_show, clustertxtcolor: gdprmap_markers[markercnt]['geojson-textcolor']
				});

				// dummy marker
	    		marker = 'geojson';
				
	    	}

	    } else {

	    	// dummy marker
	    	marker = 'geojson';
	    }

	}else{

		// Default marker -> Array empty
		marker = new maplibregl.Marker({color: defaultColor, draggable: true, id: 'marker-with-popup-'+markercnt})
		.setLngLat([map.getCenter().lng, map.getCenter().lat])
		.setPopup(popup) // sets a popup on this marker
		.addTo(map);

	}

    
 	// save marker to array
    marker_arr.push(marker);


    

    // if single Marker be draggable
    if( marker != 'geojson' ) {

    	//if( gdprmap_markers[markercnt].usehtml == "true" || jQuery('[name="gdpr-marker-use-geojson-'+markercnt+'"]:checked').val() == "true" ) {

		    // Set marker center coordinates
		    jQuery('#gdpr-marker-'+this_marker+'-lat').val(map.getCenter().lat);
		    jQuery('#gdpr-marker-'+this_marker+'-lng').val(map.getCenter().lng);

		    function onDragEnd() {

		        const lngLat = marker.getLngLat();

		        jQuery('#gdpr-marker-'+this_marker+'-lat').val(lngLat.lat);
		        jQuery('#gdpr-marker-'+this_marker+'-lng').val(lngLat.lng);

		    }

		    marker.on('dragend', onDragEnd);
		    
		    

		//}
    }
    


    // radio button listeners
    // custom icon
    jQuery('.gdpr-marker-custom-icon').off();
    jQuery('.gdpr-marker-custom-icon').click(function(){

    	var marker_id = parseInt(jQuery(this).closest('.gdpr-marker-info').attr('id').replace('gdpr-marker-info-', ''));
    	
    	if( jQuery(this).val() == "false" ) {

    		// custom icon
    		jQuery(this).parent().parent().find('.gdpr-marker-info-colorpicker').css('display', 'none');
    		jQuery(this).parent().parent().find('.gdpr-marker-info-icon-selector').css('display', 'block');

    		// change marker to icon if present
    		if( marker_arr[marker_id] != 'geojson' && jQuery('#gdpr-marker-'+marker_id+'-icon').val() != "" && load_blocker == false ) {

    			// parse JSON
    			var icondataprev = JSON.parse(jQuery('#gdpr-marker-'+marker_id+'-icon').val());

    			// Update marker HTML
        		marker_arr[marker_id].getElement().innerHTML = '<img src="'+icondataprev.url+'" width="'+icondataprev.width+'px" height="'+icondataprev.height+'px" />';
        		marker_arr[marker_id].getElement().style.width = icondataprev.width+'px';
				marker_arr[marker_id].getElement().style.height = icondataprev.height+'px';
    		}

  
  			// GeoJSON
    		if( marker_arr[marker_id] == 'geojson' && jQuery('#gdpr-marker-'+marker_id+'-icon').val() != "" && load_blocker == false ) {

    			// parse JSON
    			var icondataprev = JSON.parse(jQuery('#gdpr-marker-'+marker_id+'-icon').val());

    			// find layerID
				var layerid = "";
				gdprmap_layers.forEach(function(layer, ind){

					if(layer.markerid == marker_id) {
						layerid = layer.layerid;
						return true;
					}
					
				});

				gdprmap_update_custom_marker(layerid, icondataprev.url, 0);
    		}
    	}

    	if( jQuery(this).val() == "true" ) {

    		// default marker
    		jQuery(this).parent().parent().find('.gdpr-marker-info-colorpicker').css('display', 'block');
    		jQuery(this).parent().parent().find('.gdpr-marker-info-icon-selector').css('display', 'none');

    		// get new Color
    		var newColor = jQuery('#gdpr-marker-info-'+marker_id+' .gdpr-marker-info-colorpicker .gdpr-marker-info-colorfield').val();

    		// No GeoJSON
    		if( marker_arr[marker_id] != 'geojson' && load_blocker == false ) {


	    		// Default marker
				var default_marker = gdprmap_create_default_marker_svg(newColor).svg;


				marker_arr[marker_id].getElement().style.backgroundImage = '';
				marker_arr[marker_id].getElement().style.width = defaultWidth+'px';
				marker_arr[marker_id].getElement().style.height = defaultHeight+'px';
				marker_arr[marker_id].getElement().innerHTML = default_marker;				

			}

    		
    		// GeoJSON
    		if( marker_arr[marker_id] == 'geojson' && load_blocker == false ) {

    			// find layerID
				var layerid = "";
				gdprmap_layers.forEach(function(layer, ind){

					if(layer.markerid == marker_id) {
						layerid = layer.layerid;
						return true;
					}
					
				});

				var svgUrl = gdprmap_create_default_marker_svg(newColor).url;

				createImageBitmapFromSVG(svgUrl).then(imageBitmap => {

					if (map.hasImage('M' + layerid)) {
				     	map.removeImage('M' + layerid);
				    }

				    map.addImage('M' + layerid, imageBitmap);
				    map.redraw();

				}).catch(error => {
				    console.error('Fehler beim Erstellen des ImageBitmap:', error);
				});
    		}
    	}

    });


    // geojson
    jQuery('.gdpr-marker-use-geojson').off();
    jQuery('.gdpr-marker-use-geojson').click(function(){

    	var marker_id = parseInt(jQuery(this).closest('.gdpr-marker-info').attr('id').replace('gdpr-marker-info-', ''));


    	if( jQuery(this).val() == "false" ) {

    		// Use GEOJSON
    		jQuery(this).parent().parent().find('.gdpr-marker-info-geojson-selector').css('display', 'flex');
    		jQuery(this).parent().parent().parent().parent().find('.gdpr-marker-right').css('display', 'none');
    		jQuery(this).parent().parent().parent().parent().addClass('geojson-active');


    		// Reload Marker type button
	    	if( jQuery('#gdpr-marker-'+marker_id+'-geojson').val() != "" && load_blocker == false && marker_arr[marker_id] != 'geojson' ) {

	    		jQuery('#gdpr-marker-'+marker_id+' .gdprmap-marker-force-reload button').text('apply changes');
	    		jQuery('#gdpr-marker-'+marker_id+' .gdprmap-marker-force-reload button').attr('data-reload', 'true');
	    		jQuery('#gdpr-marker-'+marker_id+' .gdprmap-marker-force-reload button').css('display', 'inline-block');
	    	}else{
	    		jQuery('#gdpr-marker-'+marker_id+' .gdprmap-marker-force-reload button').css('display', 'none');
	    	}
    	}

    	if( jQuery(this).val() == "true" ) {

    		// Use HTML
    		jQuery(this).parent().parent().find('.gdpr-marker-info-geojson-selector').css('display', 'none');
    		jQuery(this).parent().parent().parent().parent().find('.gdpr-marker-right').css('display', 'block');
    		jQuery(this).parent().parent().parent().parent().removeClass('geojson-active');

    		// Reload Marker type button
	    	if( jQuery('#gdpr-marker-'+marker_id+'-geojson').val() != "" && load_blocker == false && marker_arr[marker_id] == 'geojson' ) {

	    		jQuery('#gdpr-marker-'+marker_id+' .gdprmap-marker-force-reload button').text('apply changes');
	    		jQuery('#gdpr-marker-'+marker_id+' .gdprmap-marker-force-reload button').attr('data-reload', 'true');
	    		jQuery('#gdpr-marker-'+marker_id+' .gdprmap-marker-force-reload button').css('display', 'inline-block');
	    	}else{
	    		jQuery('#gdpr-marker-'+marker_id+' .gdprmap-marker-force-reload button').text('goto marker');
	    		jQuery('#gdpr-marker-'+marker_id+' .gdprmap-marker-force-reload button').attr('data-reload', '');
	    	}

    	}

    });


    // cluster marker
    jQuery('.gdpr-marker-cluster-geojson').off();
    jQuery('.gdpr-marker-cluster-geojson').click(function(){

    	var marker_id = parseInt(jQuery(this).closest('.gdpr-marker-info').attr('id').replace('gdpr-marker-info-', ''));

    	if( jQuery(this).val() == "true" ) {

    		jQuery(this).parent().parent().find('.gdpr-marker-info-geojson-selector-section-1').css('display', 'block');

    		
    		// Show Cluster layers
    		// GeoJSON
    		if( marker_arr[marker_id] == 'geojson' && load_blocker == false ) {

    			// find layerID
				var layerid = "";
				var sourceid = "";
				var source;
				var clusterjson = "";
				gdprmap_layers.forEach(function(layer, ind){

					if(layer.markerid == marker_id) {
						layerid = layer.layerid;
						sourceid = layer.sourceid;
						source = layer.source;
						clusterjson = layer.clusterjson;
						return true;
					}
					
				});

				// remove unclustered layers
				if (map.getLayer('clusters'+layerid)) {
			        map.removeLayer('clusters'+layerid);
			    }
			    if (map.getLayer('cluster-count'+layerid)) {
			        map.removeLayer('cluster-count'+layerid);
			    }
			    if (map.getLayer(layerid)) {
			        map.removeLayer(layerid);
			    }
			    if (map.getSource(sourceid)) {
			        map.removeSource(sourceid);
			    }

			    // enable cluster
			    source.cluster = true;
	            source.clusterMaxZoom = 14;
	            source.clusterRadius = 50;
	            var marker_min = source.clusterMaxZoom;

	            
	            // add source
	            map.addSource(sourceid, source);

	            // add cluster layer
	            // Cluster circles
			  	map.addLayer({
			        id: 'clusters'+layerid,
			        type: 'circle',
			        source: sourceid,
			        filter: ['has', 'point_count'],
			        paint: clusterjson,
			        maxzoom: marker_min
			    });


			  	// Cluster count
		        map.addLayer({
			        id: 'cluster-count'+layerid,
			        type: 'symbol',
			        source: sourceid,
			        filter: ['has', 'point_count'],
			        layout: {
			            'text-field': '{point_count_abbreviated}',
			            'text-font': ['Open Sans Regular'],
			            'text-size': 12
			        },
			        maxzoom: marker_min,
				    paint: {
				        'text-color': jQuery('#gdpr-marker-'+marker_id+' .gdpr-marker-cluster-geojson-textcolor').val()
				    }
			    });

			    // add marker
			    if( jQuery('[name="gdpr-marker-custom-icon-'+marker_id+'"]:checked').val() == "false" && jQuery('#gdpr-marker-'+marker_id+'-icon').val() != "" ) {
  		
			    	var tmpimgurl = JSON.parse(jQuery('#gdpr-marker-'+marker_id+'-icon').val());

			    	// Custom icon marker
			  		gdprmap_create_custom_marker(layerid, sourceid, tmpimgurl.url, marker_min, map)

			  		// set new offet
			  		default_offset = parseInt(tmpimgurl.height*0.61);

			  	}else{

			  		// default marker
			  		gdprmap_create_default_marker(layerid, sourceid, jQuery('#gdpr-marker-'+marker_id+' .gdpr-marker-info-colorfield').val(), marker_min, map);		
			  	}

    		}
    		
    	}

    	if( jQuery(this).val() == "false" ) {

    		jQuery(this).parent().parent().find('.gdpr-marker-info-geojson-selector-section-1').css('display', 'none');

    		if( marker_arr[marker_id] == 'geojson' && load_blocker == false ) {

    			// find layerID
				var layerid = "";
				var sourceid = "";
				var source;
				gdprmap_layers.forEach(function(layer, ind){

					if(layer.markerid == marker_id) {
						layerid = layer.layerid;
						sourceid = layer.sourceid;
						source = layer.source;
						return true;
					}
					
				});

				// remove unclustered layers
				if (map.getLayer('clusters'+layerid)) {
			        map.removeLayer('clusters'+layerid);
			    }
			    if (map.getLayer('cluster-count'+layerid)) {
			        map.removeLayer('cluster-count'+layerid);
			    }
			    if (map.getLayer(layerid)) {
			        map.removeLayer(layerid);
			    }
			    if (map.getSource(sourceid)) {
			        map.removeSource(sourceid);
			    }

			    // remove cluster elements
			    Object.keys(source).forEach(key => {
				    if (key.startsWith('cluster')) {
				        delete source[key];
				    }
				});

			    // add source
	            map.addSource(sourceid, source);

	            // add marker
			    if( jQuery('[name="gdpr-marker-custom-icon-'+marker_id+'"]:checked').val() == "false" && jQuery('#gdpr-marker-'+marker_id+'-icon').val() != "" ) {
  		
			    	var tmpimgurl = JSON.parse(jQuery('#gdpr-marker-'+marker_id+'-icon').val());

			    	// Custom icon marker
			  		gdprmap_create_custom_marker(layerid, sourceid, tmpimgurl.url, 0, map)

			  		// set new offet
			  		default_offset = parseInt(tmpimgurl.height)*0.61;

			  	}else{

			  		// default marker
			  		gdprmap_create_default_marker(layerid, sourceid, jQuery('#gdpr-marker-'+marker_id+' .gdpr-marker-info-colorfield').val(), 0, map);		
			  	}

			}
    	}

    });


    // Cluster activate row
    jQuery('.gdpr-marker-cluster-geojson-active-1').off();
    jQuery('.gdpr-marker-cluster-geojson-active-2').off();
    jQuery('.gdpr-marker-cluster-geojson-active-3').off();

    jQuery('.gdpr-marker-cluster-geojson-active-1').click(function(){
    	jQuery(this).prop('checked', true);
    	return false;
    });

    jQuery('.gdpr-marker-cluster-geojson-active-2').click(function(){
    	jQuery(this).prop('checked', true);
    	jQuery(this).parent().parent().next().find('.gdpr-marker-cluster-geojson-active-3').prop('checked', false);
    });


    // Cluster radius, threshold, level listener
    jQuery('.gdpr-marker-info-geojson-selector-section-1 table .gdpr-marker-cluster-geojson-active-3, .gdpr-marker-info-geojson-selector-section-1 table .gdpr-marker-cluster-geojson-radius, .gdpr-marker-info-geojson-selector-section-1 table .gdpr-marker-cluster-geojson-threshold').off('change')
	jQuery('.gdpr-marker-info-geojson-selector-section-1 table .gdpr-marker-cluster-geojson-active-3, .gdpr-marker-info-geojson-selector-section-1 table .gdpr-marker-cluster-geojson-radius, .gdpr-marker-info-geojson-selector-section-1 table .gdpr-marker-cluster-geojson-threshold').change(function(){

		

		var marker_id = parseInt(jQuery(this).closest('.gdpr-marker-info').attr('id').replace('gdpr-marker-info-', ''));
		// find layerID
		var layerid = "";
		gdprmap_layers.forEach(function(layer, ind){

			if(layer.markerid == marker_id) {
				layerid = layer.layerid;
				return true;
			}
			
		});

		var tmp_paint_radius = [
		        'step',
		        ['get', 'point_count'],
		        parseInt(jQuery('#gdpr-marker-'+marker_id+' .gdpr-marker-info-geojson-selector-section-1 table tr:nth-child(1) .gdpr-marker-cluster-geojson-radius').val()),
		        parseInt(jQuery('#gdpr-marker-'+marker_id+' .gdpr-marker-info-geojson-selector-section-1 table tr:nth-child(1) .gdpr-marker-cluster-geojson-threshold').val()),
		        parseInt(jQuery('#gdpr-marker-'+marker_id+' .gdpr-marker-info-geojson-selector-section-1 table tr:nth-child(2) .gdpr-marker-cluster-geojson-radius').val())
		    ];

		
		if( jQuery('#gdpr-marker-'+marker_id+' .gdpr-marker-info-geojson-selector-section-1 table tr:nth-child(3) input[type="checkbox"]').prop('checked') == true ) {
			
			tmp_paint_radius.push( parseInt(jQuery('#gdpr-marker-'+marker_id+' .gdpr-marker-info-geojson-selector-section-1 table tr:nth-child(2) .gdpr-marker-cluster-geojson-threshold').val()) );
			tmp_paint_radius.push( parseInt(jQuery('#gdpr-marker-'+marker_id+' .gdpr-marker-info-geojson-selector-section-1 table tr:nth-child(3) .gdpr-marker-cluster-geojson-radius').val()) );
			
		}


		// Circle color
		var tmp_paint_color = [
		        'step',
		        ['get', 'point_count'],
		        jQuery('#gdpr-marker-'+marker_id+' .gdpr-marker-info-geojson-selector-section-1 table tr:nth-child(1) .gdpr-marker-cluster-geojson-color').val(),
		        parseInt(jQuery('#gdpr-marker-'+marker_id+' .gdpr-marker-info-geojson-selector-section-1 table tr:nth-child(1) .gdpr-marker-cluster-geojson-threshold').val()),
		        jQuery('#gdpr-marker-'+marker_id+' .gdpr-marker-info-geojson-selector-section-1 table tr:nth-child(2) .gdpr-marker-cluster-geojson-color').val()
		    ];

		
		if( jQuery('#gdpr-marker-'+marker_id+' .gdpr-marker-info-geojson-selector-section-1 table tr:nth-child(3) input[type="checkbox"]').prop('checked') == true ) {
			
			tmp_paint_color.push( parseInt(jQuery('#gdpr-marker-'+marker_id+' .gdpr-marker-info-geojson-selector-section-1 table tr:nth-child(2) .gdpr-marker-cluster-geojson-threshold').val()) );
			tmp_paint_color.push( jQuery('#gdpr-marker-'+marker_id+' .gdpr-marker-info-geojson-selector-section-1 table tr:nth-child(3) .gdpr-marker-cluster-geojson-color').val() );
			
		}
		


		map.setPaintProperty('clusters'+layerid, 'circle-radius', tmp_paint_radius);
		map.setPaintProperty('clusters'+layerid, 'circle-color', tmp_paint_color);
	});
	

	// del-button
	jQuery('.del-button').off('blur');
		jQuery('.del-button').on('blur', function(){
	    var that = jQuery(this);
	    setTimeout(function(){
	        that.next().css('display', 'none');
	    }, 300);
	});

    markercnt++;
}


/**
 * change marker type
 */
function gdprmap_jump_to_marker(mid, element) {

	if( jQuery(element).attr('data-reload') == 'true' ) {
		jQuery('#save-post').click();
		return false;
	}

	map.setCenter([ parseFloat(jQuery('#gdpr-marker-'+mid+'-lng').val()), parseFloat(jQuery('#gdpr-marker-'+mid+'-lat').val())]);
	jQuery("#gdprmap_center").val( jQuery('#gdpr-marker-'+mid+'-lat').val()+","+jQuery('#gdpr-marker-'+mid+'-lng').val() );

	
	jQuery('html, body').animate({
                    scrollTop: jQuery('#gdprmap_metabox').offset().top
                }, 500);
}


/**
 * delete marker
 */
function gdprmap_delete_marker(mid) {

	if( marker_arr[mid] != 'geojson' ) {
		marker_arr[mid].remove();
		popup_arr[mid].remove();
	}
	
	jQuery('#gdpr-marker-'+mid).remove();
}



/**
 * checkbox listeners
 */
jQuery(document).ready(function(){

	// double click zoom
	jQuery('#gdprmap_doubleclick').click(function(){

	   	if( jQuery(this).prop('checked') ) {
	   		map.doubleClickZoom.enable();
	   	}else{
	   		map.doubleClickZoom.disable();
	   	}
	});

   	// scroll zoom
   	jQuery('#gdprmap_scroll').click(function(){

	   	if( !jQuery(this).prop('checked') ) {
	   		map.scrollZoom.enable();
	   	}else{
	   		map.scrollZoom.disable();
	   	}
	});

   	// add zoom and rotation controls to the map.
   	jQuery('#gdprmap_zoom_controll').click(function(){

		if( !jQuery(this).prop('checked') ) {
	    	map_controll_zoom = new maplibregl.NavigationControl();
   			map.addControl(map_controll_zoom);
		}else{
			map.removeControl(map_controll_zoom); 
		}
	});

  	// fullscreen
	jQuery('#gdprmap_fullscreen_controll').click(function(){

	   	if( jQuery(this).prop('checked') ) {
	   		map_controll_full = new maplibregl.FullscreenControl();
			map.addControl(map_controll_full);
	   	}else{
			map.removeControl(map_controll_full); 
		}
	});

    // Show Geolocator
    jQuery('#gdprmap_geolocator').click(function(){

	    if( jQuery(this).prop('checked') ) {

		    map_controll_geo = new maplibregl.GeolocateControl({
		        positionOptions: {
		            enableHighAccuracy: true
		        },
		        trackUserLocation: true
		    });

    		map.addControl(map_controll_geo);

		}else{
			map.removeControl(map_controll_geo); 
		}
	});

});


/**
 * Style selector
 */
jQuery('.gdprmap-style-selector .gdprmap-style-sel').click(function(){

	jQuery('.gdprmap-style-selector .gdprmap-style-sel').removeClass('selected');
	jQuery(this).addClass('selected');

	jQuery('#gdprmap_style').val(jQuery(this).attr('data-style'));

	// set new style
	map.setStyle('https://map02.gdpr-map.eu/styles/'+jQuery(this).attr('data-style')+'/style.json?key='+WPURLS.gdprmap_api_key);

	// reload geojson layer
	map.on('styledata', function gdprmap_restoreLayers() {

        jQuery('.gdpr-marker-cluster-geojson:checked').click();

        map.off('styledata', gdprmap_restoreLayers);
    });
});



/**
 * marker custom icon
 */
var gdprmap_file_frame_icon;
var wp_media_post_id = wp.media.model.settings.post.id; // Store the old id
var gdpr_marker_id;
function gdprmap_custom_marker_style(mid) {

	var set_to_post_id;
	gdpr_marker_id = mid;

	// set to last selected attachement id
	if( jQuery('#gdpr-marker-'+mid+'-icon').val() != "" ) {
		set_to_post_id = JSON.parse(jQuery('#gdpr-marker-'+mid+'-icon').val()).id;
	}


	// If the media frame already exists, reopen it.
    if ( gdprmap_file_frame_icon ) {
        // Set the post ID to what we want
        gdprmap_file_frame_icon.uploader.uploader.param( 'post_id', set_to_post_id );
        // Open frame
        gdprmap_file_frame_icon.open();
        return;
    } else {
        // Set the wp.media post id so the uploader grabs the ID we want when initialised
        wp.media.model.settings.post.id = set_to_post_id;
    }
    // Create the media frame.
    gdprmap_file_frame_icon = wp.media.frames.gdprmap_file_frame_icon = wp.media({
        title: 'Select marker icon',
        button: {
            text: 'Use this file',
        },
        'library': {
            type: 'image'
        },
        multiple: false // Set to true to allow multiple files to be selected
    });
    // When an image is selected, run a callback.
    gdprmap_file_frame_icon.on( 'select', function() {

        // We set multiple to false so only get one image from the uploader
        attachment = gdprmap_file_frame_icon.state().get('selection').first().toJSON();

        var attachment_short = {id: attachment.id, url: attachment.url, height: attachment.height, width: attachment.width};

        // Do something with attachment.id and/or attachment.url here
        jQuery( '#gdpr-marker-info-'+gdpr_marker_id ).find('.gdpr-marker-info-icon').remove();
        jQuery( '#gdpr-marker-info-'+gdpr_marker_id+' .gdpr-marker-info-icon-selector' ).append( '<div class="gdpr-marker-info-icon"><img src="'+attachment.url+'" /></div>' );
        jQuery( '#gdpr-marker-'+gdpr_marker_id+'-icon' ).val( JSON.stringify(attachment_short) );
        // Restore the main post ID
        wp.media.model.settings.post.id = wp_media_post_id;

        // Update marker HTML
        if( marker_arr[gdpr_marker_id] != 'geojson' ) {
        	marker_arr[gdpr_marker_id].getElement().innerHTML = '<img src="'+attachment.url+'" width="'+attachment.width+'px" height="'+attachment.height+'px" />';
        }else{
        	jQuery('[name="gdpr-marker-cluster-geojson-'+gdpr_marker_id+'"]:checked').click();
        }
        
    });
    
    // Finally, open the modal
    gdprmap_file_frame_icon.open();
}




// select geojson
var gdprmap_file_frame_geojson;
function gdprmap_upload_geojson(mid) {

	var set_to_post_id;
	gdpr_marker_id = mid;

	// set to last selected attachement id
	if( jQuery('#gdpr-marker-'+mid+'-geojson').val() != "" ) {
		set_to_post_id = JSON.parse(jQuery('#gdpr-marker-'+mid+'-geojson').val()).id;
	}


	// If the media frame already exists, reopen it.
    if ( gdprmap_file_frame_geojson ) {
        // Set the post ID to what we want
        gdprmap_file_frame_geojson.uploader.uploader.param( 'post_id', set_to_post_id );
        // Open frame
        gdprmap_file_frame_geojson.open();
        return;
    } else {
        // Set the wp.media post id so the uploader grabs the ID we want when initialised
        wp.media.model.settings.post.id = set_to_post_id;
    }
    // Create the media frame.
    gdprmap_file_frame_geojson = wp.media.frames.gdprmap_file_frame_geojson = wp.media({
        title: 'Select GEOJSON file',
        button: {
            text: 'Use this file',
        },
        'library': {
            type: 'application/json'
        },
        multiple: false // Set to true to allow multiple files to be selected
    });
    // When an image is selected, run a callback.
    gdprmap_file_frame_geojson.on( 'select', function() {

        // We set multiple to false so only get one image from the uploader
        attachment = gdprmap_file_frame_geojson.state().get('selection').first().toJSON();

        var attachment_short = {id: attachment.id, url: attachment.url, title: attachment.title};

        // Do something with attachment.id and/or attachment.url here
        jQuery( '#gdpr-marker-info-'+gdpr_marker_id ).find('.gdpr-marker-info-geojson span').remove();
        jQuery( '#gdpr-marker-info-'+gdpr_marker_id+' .gdpr-marker-info-geojson' ).append( '<span>'+attachment.title+'</span>' );
        jQuery( '#gdpr-marker-'+gdpr_marker_id+'-geojson' ).val( JSON.stringify(attachment_short) );
        // Restore the main post ID
        wp.media.model.settings.post.id = wp_media_post_id;

        // Show apply changes Button
        jQuery( '#save-post' ).click();
    });
    
    // Finally, open the modal
    gdprmap_file_frame_geojson.open();
}



/**
 * onload create marker
 */
function gdprmap_add_edit_markers() {

	var all_markers = gdprmap_markers;


	if( all_markers == '' ) {
		return true;
	}


	all_markers.reverse();
	
	var cnt = 0;
	while( cnt < all_markers.length) {

		gdprmap_add_marker();

		jQuery('#gdpr-marker-'+(markercnt-1)+'-name').val(all_markers[cnt].name);
    	jQuery('#gdpr-marker-'+(markercnt-1)+'-html').val(all_markers[cnt].html);
    	jQuery('#gdpr-marker-'+(markercnt-1)+'-lat').val(all_markers[cnt].lat);
    	jQuery('#gdpr-marker-'+(markercnt-1)+'-lng').val(all_markers[cnt].lng);
    	jQuery('#gdpr-marker-'+(markercnt-1)+'-geojson').val(all_markers[cnt].geojson);
    	jQuery('#gdpr-marker-'+(markercnt-1)+'-icon').val(all_markers[cnt].icon);

    	// add icon
    	if(all_markers[cnt].icon != "") {
    		var this_marker_icon = JSON.parse(all_markers[cnt].icon);
    		jQuery( '#gdpr-marker-info-'+(markercnt-1)+' .gdpr-marker-info-icon' ).append( '<img src="'+this_marker_icon.url+'" />' );
    	}

    	// add geojson
    	if(all_markers[cnt].geojson != "") {
    		var this_marker_geojson = JSON.parse(all_markers[cnt].geojson);
	    	jQuery( '#gdpr-marker-info-'+(markercnt-1)+' .gdpr-marker-info-geojson' ).append( '<span>'+this_marker_geojson.title+'</span>' );
	    }


	    // use custom icon
	    jQuery('#gdpr-marker-info-'+(markercnt-1)+' .gdpr-marker-info-custom-icon [value="'+all_markers[cnt].defaulticon+'"]').click();

	    // use geojson
	    jQuery('[name="gdpr-marker-use-geojson-'+(markercnt-1)+'"][value="'+all_markers[cnt].usehtml+'"]').click();

	    // use cluster
	    jQuery('[name="gdpr-marker-cluster-geojson-'+(markercnt-1)+'"][value="'+all_markers[cnt].clustergeojson+'"]').click();

	    // cluster options
	    if( all_markers[cnt].geojsonactive2 == "2" ) {
	    	jQuery('#gdpr-marker-info-'+(markercnt-1)+' input[value="2"]').prop('checked', true);
	    }
	    if( all_markers[cnt].geojsonactive3 == "3" ) {
	    	jQuery('#gdpr-marker-info-'+(markercnt-1)+' input[value="3"]').prop('checked', true);
	    }

	    var optcnt = 1;
	    while( optcnt <= 3 ) {

	    	jQuery('#gdpr-marker-info-'+(markercnt-1)+' .gdpr-marker-info-geojson-selector-section-1 tbody tr:nth-child('+optcnt+') .gdpr-marker-cluster-geojson-radius').val( all_markers[cnt]['geojsonradius'+optcnt]);
	    	optcnt++;
	    }

    	popup_arr[markercnt-1].setHTML( all_markers[cnt].html );

		cnt++;
	}
}


