/**
 * on document ready init all maps on page
 */
jQuery(document).ready(function(){

	jQuery('.gdprmaps-front').each(function(){

		var mid = jQuery(this).attr("data-mid");
		gdprmap_init_map(mid);
	});
});


/**
 * init map
 */
function gdprmap_init_map(mid) {


	// map data
	var splice = window['gdprmap_'+mid].gdprmap_center.split(',');
	var center = [ [parseFloat(splice[1])], [parseFloat(splice[0])] ];
	var zoom = window['gdprmap_'+mid].gdprmap_zoom;
	var doubleclick = false;
	doubleclick = window['gdprmap_'+mid].gdprmap_doubleclick;
	var bearing = window['gdprmap_'+mid].gdprmap_bearing;
	var pitch = window['gdprmap_'+mid].gdprmap_pitch;
	var style = window['gdprmap_'+mid].gdprmap_style;
	var geocoder = window['gdprmap_'+mid].gdprmap_searchbox;
	var geolocator = window['gdprmap_'+mid].gdprmap_geolocator;
	var fixedmap = window['gdprmap_'+mid].gdprmap_fixed_map;


	var gestures = null;


    if( window['gdprmap_'+mid].gdprmap_scroll == 'true' ) {
		gestures = {
	        windowsHelpText: "Use Ctrl + scroll to zoom the map",
			macHelpText: "Use ⌘ + scroll to zoom the map",
			mobileHelpText: "Use two fingers to move the map",
	    };
	}


	var map_style = 'https://map02.gdpr-map.eu/styles/'+style+'/style.json?key='+WPURLS.gdprmap_api_key;


	gdprmap_all_maps[mid] = new maplibregl.Map({
	  container: 'gdprmap-front-'+mid,
	  bearing: bearing,
	  pitch: pitch,
	  center: center,
	  zoom: zoom,
	  doubleClickZoom: doubleclick,
	  minZoom: 3,
	  style: map_style,
	  //maxBounds: bounds,
	  attributionControl: false,
	  //cooperativeGestures: gestures,
	  scrollZoom: true
	});


	if( geocoder == 'true' ) {

		jQuery('#gdprmap-front-'+mid).parent().prepend('<div class="gdprmap-geolocation" id="gdprmap-geolocation-'+mid+'"><input class="gdprmap-geolocation-autocomplete" type="text" autocomplete="off" id="gdprmap-geolocation-autocomplete" onkeyup="gdprmap_geolocation_autocomplete(jQuery(this)); return false;" onkeydown="return gdprmap_geolocation_autocomplete_enter();" placeholder="Suche" /><button class="button button-primary button-large" onclick="gdprmap_geolocation_autocomplete();return false;"><span class="dashicons dashicons-search"></span></button></div>');

		gdprmap_all_maps[mid].on('movestart', function(){
			jQuery('.gdprmap-geolocation-autocomplete-tbl').css('display', 'none');
			jQuery('.gdprmap-geolocation-autocomplete').removeClass('open');
		});

		gdprmap_all_maps[mid].on('dragstart', function(){
			jQuery('.gdprmap-geolocation-autocomplete-tbl').css('display', 'none');
			jQuery('.gdprmap-geolocation-autocomplete').removeClass('open');
		});
	}


	

	// check for scroll wheel
	if( window['gdprmap_'+mid].gdprmap_scroll == 'true' ) {
		gdprmap_all_maps[mid].scrollZoom.disable();

		// add fullscreen listener
		/*
		document.addEventListener('fullscreenchange', (event) => {
		  if (document.fullscreenElement) {
		    
		    jQuery.each( gdprmap_all_maps, function(k,v){
		    	v.scrollZoom.enable();
		    });
		    
		  } else {
		    // leaving fullscreen mode
		    
		    jQuery.each( gdprmap_all_maps, function(k,v){
		    	if( window['gdprmap_'+k][k].gdprmap_scroll == "true" ) {
		    		v.scrollZoom.disable();
		    	}		    	
		    });
		  }
		});
		*/
	}


	// fix mobile bad behaviour
	
	if (/Android|webOS|iPhone|iPad|iPod|BlackBerry/i.test(navigator.userAgent)) {
	    gdprmap_all_maps[mid].dragPan.disable();
	    gdprmap_all_maps[mid].scrollZoom.disable();
	    gdprmap_all_maps[mid].touchPitch.disable()
	    gdprmap_all_maps[mid].on('touchstart', function(e) {
	        var oe = e.originalEvent;
	        if (oe && 'touches' in oe) {
	            if (oe.touches.length > 1) {
	                oe.stopImmediatePropagation();
	                gdprmap_all_maps[mid].dragPan.enable();
	            } else {
	                gdprmap_all_maps[mid].dragPan.disable();
	            }
	        }
	    });
	}
	

	// fullscreen controll
	if( window['gdprmap_'+mid].gdprmap_fullscreen_controll == 'true' ) {
		gdprmap_all_maps[mid].addControl(new maplibregl.FullscreenControl());
	}

	// geolocator controll
	if( window['gdprmap_'+mid].gdprmap_geolocator == 'true' ) {
		gdprmap_all_maps[mid].addControl(
	    new maplibregl.GeolocateControl({
	        positionOptions: {
	            enableHighAccuracy: true
	        },
	        trackUserLocation: true
	    })
	);
	}

	// add zoom and rotation controls to the map.
	if( window['gdprmap_'+mid].gdprmap_zoom_controll != 'true' ) {
    	gdprmap_all_maps[mid].addControl(new maplibregl.NavigationControl());
    }

    // attribution
    gdprmap_all_maps[mid].addControl(new maplibregl.AttributionControl({
		customAttribution: '<a href="https://www.gdpr-map.eu" target="_blank" title="GDPR map" aria-label="GDPR map">GDPR map</a> | <a href="https://www.openmaptiles.org/" target="_blank" title="OpenMapTiles" aria-label="OpenMapTiles">© OpenMapTiles</a> | <a href="https://www.openstreetmap.org/copyright" target="_blank" title="OpenStreetMap" aria-label="OpenStreetMap">© OpenStreetMap contributors</a>'
	}));


    gdprmap_all_maps[mid].on('load', async () => {

	    // create simple marker
		jQuery.each(window['gdprmap_'+mid].gdprmap_markers, function(k,v){

			if( v.usehtml == 'false' ) {
				return true;
			}


			var marker;
			var popup;


		    // Custom Marker Icon
		    if( v.defaulticon == "false" && v.icon != "" ) {

		    	var icon_data = JSON.parse(v.icon);

		    	const pointjson = {
			        'type': 'FeatureCollection',
			        'features': [
			            {
			                'type': 'Feature',
			                'properties': {
			                    'iconSize': [icon_data.width, icon_data.height],
			                    'iconUrl': icon_data.url
			                },
			                'geometry': {
			                    'type': 'Point',
			                    'coordinates': [v.lng, v.lat]
			                }
			            }
			        ]
			    };


			    // create a DOM element for the marker
			    const el = document.createElement('div');
			    el.className = 'custommarkericon';
			    el.style.backgroundImage = 'url('+pointjson.features[0].properties.iconUrl+')';
			    el.style.width = `${pointjson.features[0].properties.iconSize[0]}px`;
			    el.style.height = `${pointjson.features[0].properties.iconSize[1]}px`;

			    marker = new maplibregl.Marker({element: el})
			    .setLngLat(pointjson.features[0].geometry.coordinates)
			    .addTo(gdprmap_all_maps[mid]);

		    }else{

		    	// Default Marker Icon when Icon is missing
		    	marker = new maplibregl.Marker({color: v.markercolor})
			    .setLngLat([v.lng, v.lat])
			    .addTo(gdprmap_all_maps[mid]);
		    }



		    // HTML Popup
		    if( v.html.trim() != '' ) {
		    	const popup = new maplibregl.Popup({ offset: 25 }).setHTML(v.html);
		    	marker.setPopup(popup);

		    	const markerElement = marker.getElement();

			    // hover markers change cursor to pointer
				markerElement.addEventListener('mouseenter', () => {
					jQuery(markerElement).css('cursor', 'pointer');
				});
				markerElement.addEventListener('mouseleave', () => {
					jQuery(markerElement).css('cursor', '');
				});
		    }


		});


		// add zoom and rotation controls to the map.
		if( fixedmap == 'true' ) {
	    	gdprmap_all_maps[mid].touchZoomRotate.disable();
	    	gdprmap_all_maps[mid].keyboard.disable();
	    	gdprmap_all_maps[mid].boxZoom.disable();
	    	gdprmap_all_maps[mid].dragPan.disable();
	    	gdprmap_all_maps[mid].touchZoomRotate.disableRotation();
	    }
	    


	    // add geojson
	    var markercnt = 0;
		jQuery.each(window['gdprmap_'+mid].gdprmap_markers, function(k,v){

			if( v.usehtml == 'true' ) {
				return true;
			}

			var marker_min = 0;

			var geojson = JSON.parse(v.geojson);

			if( v.icon != "" ) {
				var markericon = JSON.parse(v.icon);
			}
			

			var marker;
			var popup;
			var subsrc = { type: 'geojson', data: geojson.url };


				// extend source properties when clutering
			if( v.clustergeojson == 'true' ) {

				subsrc.cluster = true;
	            subsrc.clusterMaxZoom = 14;
	            subsrc.clusterRadius = 50;
	            marker_min = subsrc.clusterMaxZoom;
			}
		  	
		  	// load source
		  	gdprmap_all_maps[mid].addSource('source'+markercnt, subsrc);


		  	// add cluster layers
		  	if( v.clustergeojson == 'true' ) {

		  		var this_paint = {
				    'circle-color': [
				        'step',
				        ['get', 'point_count'],
				        v.geojsonclustercolor1,
				        parseInt(v.geojsonthreshold1)
				    ],
				    'circle-radius': [
				        'step',
				        ['get', 'point_count'],
				        parseInt(v.geojsonradius1),
				        parseInt(v.geojsonthreshold1)				        
				    ]
				};

				if( v.geojsonactive2 == "2" ) {

					this_paint['circle-color'].push( v.geojsonclustercolor2 );
					this_paint['circle-radius'].push( parseInt(v.geojsonradius2) );
				}

				if( v.geojsonactive3 == "3" ) {
					
					this_paint['circle-color'].push( parseInt(v.geojsonthreshold2), v.geojsonclustercolor3 );
					this_paint['circle-radius'].push( parseInt(v.geojsonthreshold2), parseInt(v.geojsonradius3) );
				}



			  	// Cluster circles
			  	gdprmap_all_maps[mid].addLayer({
			        id: 'clusters'+markercnt,
			        type: 'circle',
			        source: 'source'+markercnt,
			        filter: ['has', 'point_count'],
			        paint: this_paint,
			        maxzoom: marker_min
			    });


			  	// Cluster count
		        gdprmap_all_maps[mid].addLayer({
			        id: 'cluster-count'+markercnt,
			        type: 'symbol',
			        source: 'source'+markercnt,
			        filter: ['has', 'point_count'],
			        layout: {
			            'text-field': '{point_count_abbreviated}',
			            'text-font': ['Open Sans Regular'],
			            'text-size': 12
			        },
			        maxzoom: marker_min,
				    paint: {
				        'text-color': v['geojson-textcolor']
				    }
			    });
			}



		    // zoom on click
		    (function(currentMarkercnt) {
		        gdprmap_all_maps[mid].on('click', 'clusters' + currentMarkercnt, async (e) => {

		            const features = gdprmap_all_maps[mid].queryRenderedFeatures(e.point, {
		                layers: ['clusters' + currentMarkercnt]
		            });
		            const clusterId = features[0].properties.cluster_id;
		            const zoom = await gdprmap_all_maps[mid].getSource('source' + currentMarkercnt).getClusterExpansionZoom(clusterId);
		            gdprmap_all_maps[mid].easeTo({
		                center: features[0].geometry.coordinates,
		                zoom
		            });
		        });
		    })(markercnt);

		    // Change cursor to pointer
		    gdprmap_all_maps[mid].on('mouseenter', 'clusters'+markercnt, () => {
		        gdprmap_all_maps[mid].getCanvas().style.cursor = 'pointer';
		    });
		    gdprmap_all_maps[mid].on('mouseleave', 'clusters'+markercnt, () => {
		        gdprmap_all_maps[mid].getCanvas().style.cursor = '';
		    });





		  	// Add layer with markers
		  	var default_offset = 25;
		  	if( v.defaulticon == "true" ) {

		  		// default marker
		  		gdprmap_create_default_marker('mrk'+markercnt, 'source'+markercnt, v.markercolor, marker_min, gdprmap_all_maps[mid]);		  		


		  	}else{
		  		
		  		// Custom icon marker
		  		gdprmap_create_custom_marker('mrk'+markercnt, 'source'+markercnt, markericon.url, marker_min, gdprmap_all_maps[mid])

		  		// set new offet
		  		default_offset = parseInt(markericon.height*0.61);
		  	}


		  	// Change cursor to pointer
		    gdprmap_all_maps[mid].on('mouseenter', 'mrk'+markercnt, () => {
		        gdprmap_all_maps[mid].getCanvas().style.cursor = 'pointer';
		    });
		    gdprmap_all_maps[mid].on('mouseleave', 'mrk'+markercnt, () => {
		        gdprmap_all_maps[mid].getCanvas().style.cursor = '';
		    });


		  	// click on markers
		  	gdprmap_all_maps[mid].on('click', 'mrk'+markercnt, (e) => {

		  		console.log(e);

	            const coordinates = e.features[0].geometry.coordinates.slice();

	            if( typeof e.features[0].properties.html != 'undefined' ) {

	            	const html = e.features[0].properties.html;

		            if( html.trim() != "" ) {
		            
			            while (Math.abs(e.lngLat.lng - coordinates[0]) > 180) {
			                coordinates[0] += e.lngLat.lng > coordinates[0] ? 360 : -360;
			            }

			            new maplibregl.Popup({offset: default_offset})
			                .setLngLat(coordinates)
			                .setHTML(
			                    html
			                )
			                .addTo(gdprmap_all_maps[mid]);
			        }
	            }
	            
	        });


			markercnt++;
	    
		});

	});

	
}